package com.qls.ali.biz.service.fz.impl;

import com.alibaba.fastjson.JSON;
import com.qls.ali.biz.order.service.mybatis.DtStoreService;
import com.qls.ali.biz.service.fz.Cache;
import com.qls.core.constant.Constant;
import com.qls.core.entity.DtStore;
import com.qls.core.entity.OpOurRateplanPrice;
import com.qls.lock.RedissonLock;
import com.qls.redis.RedisUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author doyoung
 */
@Slf4j
@Service
@SuppressWarnings("unchecked")
public class CacheImpl implements Cache {

    /**
     * 分布式锁时间
     */
    private final static int TIME = 15;

    @Autowired
    private RedissonLock redissonLock;

    @Autowired
    private RedisUtil redisUtil;

    @Autowired
    private DtStoreService dtStoreService;

    @Override
    public List<OpOurRateplanPrice> getStairPrice(Integer crpId) {
        List<OpOurRateplanPrice> opOurRateplanPrices = JSON.parseArray(JSON.toJSONString(redisUtil.hashGet(Constant.Redis.CRPID_USE_CPRICE.getValue(), crpId.toString())), OpOurRateplanPrice.class);
        if (Objects.isNull(opOurRateplanPrices)) {
            return new ArrayList<>();
        }
        return opOurRateplanPrices;
    }

    @Override
    public DtStore getStore(Integer storeId) {
        DtStore dtStore = (DtStore) redisUtil.hashGet(Constant.Redis.STOREID_USER_STORE.getValue(), storeId.toString());
        if (dtStore == null) {
            dtStore = dtStoreService.getById(storeId);
            if (dtStore == null) {
                return null;
            }
            redisUtil.hashSet(Constant.Redis.STOREID_USER_STORE.getValue(), dtStore.getId().toString(), dtStore);
        }
        return dtStore;
    }

    /**
     * 删除 全局过滤缓存
     *
     * @param rpCode  String
     * @param storeId Integer
     * @return boolean
     */
    @Override
    public boolean delGlobFilterCache(String rpCode, Integer storeId) {
        String h = Constant.Redis.RP_BL_STORE.getValue();
        // 删除全局过滤缓存
        try {
            redissonLock.lock(rpCode, TIME);
            Map<String, Object> mapStoreIds = this.getGlobFilterCache(rpCode);
            if (mapStoreIds.isEmpty()) {
                return true;
            }
            List<Integer> storeIds = (List<Integer>) mapStoreIds.get("storeIds");
            if (storeIds.isEmpty()) {
                redisUtil.hashDel(Constant.Redis.RP_BL_STORE.getValue(), rpCode);
            } else {
                storeIds.remove(storeId);
                if (storeIds.isEmpty()) {
                    redisUtil.hashDel(h, rpCode);
                } else {
                    redisUtil.hashSet(h, rpCode, mapStoreIds);
                }
            }
            return true;
        } catch (Exception e) {
            log.error("店铺ID：{}， rpID：{}，删除全局过滤缓存失败，异常：{}", storeId, rpCode, e.getMessage());
            return false;
        } finally {
            if (redissonLock.isHeldByCurrentThread(rpCode)) {
                redissonLock.unlock(rpCode);
            }
        }
    }

    /**
     * 全局过滤的缓存 主要缓存 这个rp哪个店铺推送过 增量更新时需要更新对应的店铺
     *
     * @param rpCode    String
     * @param storeId   Integer
     * @param partnerId Integer
     * @return boolean
     */
    @Override
    public boolean addGlobFilterCache(String rpCode, Integer storeId, Integer partnerId) {
        // redis hash 的h
        String h = Constant.Redis.RP_BL_STORE.getValue();
        List<Integer> storeIds;
        Map<String, Object> mapStore;
        try {
            // 查询时上锁防止 并发问题
            redissonLock.lock(rpCode, TIME);
            mapStore = (Map<String, Object>) redisUtil.hashGet(h, rpCode);
            // 如果查询不出说明 没有店铺推送过这个rp 需要进行增加缓存
            if (Objects.isNull(mapStore)) {
                mapStore = new HashMap<>();
                storeIds = new ArrayList<>();
                storeIds.add(storeId);
            } else {
                // 如果查询出缓存 需要取出缓存中的 店铺主键 进行对比 若存在说明之前这个店铺推送过 不需要再向集合中添加 反之。。
                // 当店铺对应的缓存不存在时 其实全局过滤缓存一定不会存在 这个 店铺的缓存信息 防止脏数据需判断一下
                storeIds = (List<Integer>) mapStore.get("storeIds");
                if (!storeIds.contains(storeId)) {
                    storeIds.add(storeId);
                } else {
                    // 如果已经存在 不需要再进行保存 直接返回
                    return true;
                }
            }
            mapStore.put("storeIds", storeIds);
            mapStore.put("partnerId", partnerId);
            // 缓存修改
            redisUtil.hashSet(h, rpCode, mapStore);
            return true;
        } catch (Exception e) {
            log.error("店铺ID：{}， rpID：{}，增加全局过滤缓存失败，异常：{}", storeId, rpCode, e.getMessage());
            return false;
        } finally {
            if (redissonLock.isHeldByCurrentThread(rpCode)) {
                redissonLock.unlock(rpCode);
            }
        }
    }

    @Override
    public Map<String, Object> getGlobFilterCache(String rpCode) {
        Map<String, Object> mapStoreIds = (Map<String, Object>) redisUtil.hashGet(Constant.Redis.RP_BL_STORE.getValue(), rpCode);
        if (Objects.isNull(mapStoreIds)) {
            return new HashMap<>();
        }
        return mapStoreIds;
    }

    /**
     * 店铺级别的缓存 每个店铺都会对应一个
     *
     * @param roomCode  String
     * @param rpCode    String
     * @param storeId   Integer
     * @param crpIds    List<Integer>
     * @param partnerId Integer
     * @return List<Integer>
     */
    @Override
    public List<Integer> addRpStoreMap(String roomCode, String rpCode, Integer storeId, List<Integer> crpIds, Integer partnerId) {
        // 分布式锁名
        String lockName = rpCode + "_" + storeId;
        // redis hash 的h
        String h = Constant.Redis.GOODS_BL_CRP_HTL_ROOM_RP_STORE.getValue() + "_" + storeId;
        List<Integer> newCrpIds = new ArrayList<>();
        Map<String, Object> cacheMap;
        try {
            redissonLock.lock(lockName, TIME);
            // 查询之前就要进行上锁 否则很容易出现并发问题
            cacheMap = (Map<String, Object>) redisUtil.hashGet(h, rpCode);
            // 如果缓存为null 说明没有推送过 需要添加缓存
            if (Objects.isNull(cacheMap)) {
                // 新添加缓存的方法
                boolean flag = this.addGlobFilterCache(rpCode, storeId, partnerId);
                // 添加不成功 返回空的
                if (!flag) {
                    return newCrpIds;
                }
                cacheMap = new HashMap<>();
                cacheMap.put("roomCode", roomCode);
                cacheMap.put("crpIds", crpIds);
                newCrpIds.addAll(crpIds);
            } else {
                // 缓存已存在的情况 查询缓存中对应的自定义价格计划的主键
                List<Integer> cacheCrpIds = (List<Integer>) cacheMap.get("crpIds");
                for (Integer crpId : crpIds) {
                    // 如果包含这个 则跳过
                    if (cacheCrpIds.contains(crpId)) {
                        continue;
                    }
                    // 如果不包含 缓存中需要添加这个
                    cacheCrpIds.add(crpId);
                    // 这个操作是为了返回给推送rp用的 如果返回为空集合 则不进行数据库rp个数加一操作
                    newCrpIds.add(crpId);
                }
            }
            // 如果这个为空 说明历史缓存已经存了 不需要再进行修改
            if (newCrpIds.isEmpty()) {
                return newCrpIds;
            }
            redisUtil.hashSet(h, rpCode, cacheMap);
            return newCrpIds;
        } catch (Exception e) {
            log.error("店铺ID：{}，增加rpId：{}，对应的价格政策失败，异常：{}", storeId, rpCode, e.getMessage());
            return new ArrayList<>();
        } finally {
            if (redissonLock.isHeldByCurrentThread(lockName)) {
                redissonLock.unlock(lockName);
            }
        }
    }

    @Override
    public Map<String, Object> getRpStoreMap(String rpCode, Integer storeId) {
        Map<String, Object> rpStoreCache = new HashMap<>();
        try {
            Map<String, Object> map = (Map<String, Object>) redisUtil.hashGet(Constant.Redis.GOODS_BL_CRP_HTL_ROOM_RP_STORE.getValue() + "_" + storeId, rpCode);
            if (Objects.isNull(map)) {
                this.delGlobFilterCache(rpCode, storeId);
                return rpStoreCache;
            }
            List<Integer> crpIdsCache = (List<Integer>) map.get("crpIds");
            if (Objects.isNull(crpIdsCache) || crpIdsCache.isEmpty()) {
                this.delGlobFilterCache(rpCode, storeId);
                return rpStoreCache;
            }
            return map;
        } catch (Exception e) {
            log.error("店铺ID：{}，查询店铺rp缓存失败：{}，异常：{}", storeId, rpCode, e.getMessage());
            return rpStoreCache;
        }
    }

    /**
     * 删除店铺缓存对应的rp
     *
     * @param rpCode  String
     * @param storeId Integer
     * @param crpIds  List<Integer>
     * @return boolean
     */
    @Override
    public boolean delRpStoreMap(String rpCode, Integer storeId, List<Integer> crpIds) {
        // 锁
        String lockName = rpCode + "_" + storeId;
        // h
        String h = Constant.Redis.GOODS_BL_CRP_HTL_ROOM_RP_STORE.getValue() + "_" + storeId;
        try {
            redissonLock.lock(lockName, TIME);
            // 查询缓存 如果为null 说明已删除
            Map<String, Object> cacheMap = (Map<String, Object>) redisUtil.hashGet(h, rpCode);
            if (Objects.isNull(cacheMap)) {
                return true;
            }
            List<Integer> cacheCrpIds = (List<Integer>) cacheMap.get("crpIds");
            // 这种情况基本不会出现
            if (Objects.isNull(cacheCrpIds) || cacheCrpIds.isEmpty()) {
                return this.delGlobFilterCache(rpCode, storeId);
            }
            cacheCrpIds.removeAll(crpIds);
            if (cacheCrpIds.isEmpty()) {
                redisUtil.hashDel(h, rpCode);
                return this.delGlobFilterCache(rpCode, storeId);
            } else {
                redisUtil.hashSet(h, rpCode, cacheMap);
                return true;
            }
        } catch (Exception e) {
            log.error("店铺ID：{}，删除crpIds：{}，对应的价格政策失败，异常：{}", storeId, crpIds, e.getMessage());
            return false;
        } finally {
            if (redissonLock.isHeldByCurrentThread(lockName)) {
                redissonLock.unlock(lockName);
            }
        }
    }

}
