package com.qls.redis;

import cn.hutool.core.convert.Convert;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import com.google.gson.Gson;
import com.qls.core.dto.HtDailyRoom;
import com.qls.core.utils.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.*;
import org.springframework.stereotype.Component;

import java.util.*;
import java.util.concurrent.TimeUnit;

/**
 * redisTemplate封装
 *
 * @author dy
 */
@Slf4j
@Component
@SuppressWarnings("unchecked")
public class RedisUtil {

    /**
     * 默认到期
     */
    private final static long DEFAULT_EXPIRE = 60 * 60 * 24;
    /**
     * 没有到期
     */
    private final static long NOT_EXPIRE = -1;
    /**
     * gson
     */
    private final static Gson gson = new Gson();
    /**
     * 复述,模板
     */
    @Autowired
    private RedisTemplate<String, Object> redisTemplate;
    /**
     * 值操作
     */
    @Autowired
    private ValueOperations<String, String> valueOperations;
    /**
     * 散列操作
     */
    @Autowired
    private HashOperations<String, String, Object> hashOperations;
    /**
     * 列表操作
     */
    @Autowired
    private ListOperations<String, Object> listOperations;
    /**
     * 集合操作
     */
    @Autowired
    private SetOperations<String, Object> setOperations;
    /**
     * z集合操作
     */
    @Autowired
    private ZSetOperations<String, Object> zSetOperations;

    /**
     * redis 自增
     *
     * @param key
     * @return
     */
    public long incr(String key, int step, long expire) {
        long incrNum = redisTemplate.boundValueOps(key).increment(step);
//        System.out.println("剩余过期时间" + redisTemplate.boundValueOps(key).getExpire());
        if (incrNum == 1L) {
            redisTemplate.boundValueOps(key).expire(expire, TimeUnit.SECONDS);
        }
        return incrNum;
    }

    /**
     * hash set 无失效时间
     *
     * @param h
     * @param hk
     * @param value
     */
    public void hashSet(String h, String hk, Object value) {
        hashSet(h, hk, value, NOT_EXPIRE);
    }

    /**
     * hash set 自定义失效时间
     *
     * @param h
     * @param hk
     * @param value
     * @param expire
     */
    public void hashSet(String h, String hk, Object value, long expire) {
        hashOperations.put(h, hk, value);
        if (expire != NOT_EXPIRE) {
            redisTemplate.expire(h, expire, TimeUnit.SECONDS);
        }
    }

    /**
     * 批量 hash Set  无失效时间
     *
     * @param h
     * @param map
     */
    public void hashSetAll(String h, Map<? extends String, ?> map) {
        hashSetAll(h, map, NOT_EXPIRE);
    }

    /**
     * 批量 hash Set 自定义失效时间
     *
     * @param h
     * @param map
     * @param expire
     */
    public void hashSetAll(String h, Map<? extends String, ?> map, long expire) {
        hashOperations.putAll(h, map);
        if (expire != NOT_EXPIRE) {
            redisTemplate.expire(h, expire, TimeUnit.DAYS);
        }
    }

    /**
     * hash Get
     *
     * @param h
     * @param hk
     * @return
     */
    public Object hashGet(String h, String hk) {
        return hashOperations.get(h, hk);
    }

    /**
     * 批量 hashGet
     *
     * @param h
     * @param fields
     * @return
     */
    public List<Object> hashGetAll(String h, List<String> fields) {
        return hashOperations.multiGet(h, fields);
    }

    /**
     * 删除 hash
     *
     * @param h
     * @param hks
     * @return
     */
    public Long hashDel(String h, String... hks) {
        return hashOperations.delete(h, hks);
    }

    /**
     * set  自定义生效时间
     *
     * @param key
     * @param value
     * @param expire
     */
    public void set(String key, Object value, long expire) {
        valueOperations.set(key, toJson(value));
        if (expire != NOT_EXPIRE) {
            redisTemplate.expire(key, expire, TimeUnit.SECONDS);
        }
    }

    /**
     * set 默认生效时间
     *
     * @param key
     * @param value
     */
    public void set(String key, Object value) {
        set(key, value, DEFAULT_EXPIRE);
    }

    /**
     * 集合添加数据
     *
     * @param k
     * @param values
     * @return
     */
    public Long addSet(String k, Object... values) {
        return redisTemplate.opsForSet().add(k, values);
    }

    /**
     * 获取集合的所有数据
     *
     * @param k
     * @return
     */
    public <T> T getSet(String k, Class<T> clazz) {
        return (T) redisTemplate.opsForSet().members(k);
    }

    /**
     * 集合删除数据
     *
     * @param k
     * @param values
     * @return
     */
    public Long setRemove(String k, Object... values) {
        return redisTemplate.opsForSet().remove(k, values);
    }

    /**
     * 判断是否存在集合中
     *
     * @param k
     * @param value
     * @return
     */
    public boolean setExist(String k, Object value) {
        Boolean flag;
        try {
            flag = redisTemplate.opsForSet().isMember(k, value);
        } catch (Exception e) {
            log.error("redis 判断set集合是否存在 某个值出现异常 ：{}", e.toString());
            flag = false;
        }
        return flag == null ? false : flag;
    }

    /**
     * 判断是否在hash的filed中
     *
     * @param k
     * @param value
     * @return
     */
    public boolean hashExistFiled(String k, Object value) {
        return hashOperations.hasKey(k, value);
    }

    /**
     * 求差集
     *
     * @param
     * @param k
     * @return
     */
    public Set<Object> difference(String k, String otherK) {
        return redisTemplate.opsForSet().difference(k, otherK);
    }

    /**
     * 将数据放入set缓存
     *
     * @param key    键
     * @param values 值 可以是多个
     * @return 成功个数
     */
    public long sSetAdd(String key, Object... values) {
        Long count;
        try {
            count = setOperations.add(key, values);
        } catch (Exception e) {
            log.error("redis set 集合添加数据异常，k:{}, v:{}，reason:{}", key, values, e.toString());
            count = 0L;
        }
        return count == null ? 0 : count;
    }

    /**
     * get 并设置过期时间
     *
     * @param key
     * @param clazz
     * @param expire
     * @param <T>
     * @return
     */
    public <T> T get(String key, Class<T> clazz, long expire) {
        String value = valueOperations.get(key);
        if (expire != NOT_EXPIRE) {
            redisTemplate.expire(key, expire, TimeUnit.SECONDS);
        }
        return value == null ? null : fromJson(value, clazz);
    }

    /**
     * get 时间设置 不失效
     *
     * @param key
     * @param clazz
     * @param <T>
     * @return
     */
    public <T> T get(String key, Class<T> clazz) {
        return get(key, clazz, NOT_EXPIRE);
    }

    /**
     * 删除缓存
     *
     * @param key
     */
    public void delete(String key) {
        redisTemplate.delete(key);
    }

    /**
     * Object转成JSON数据
     */
    private String toJson(Object object) {
        if (object instanceof Integer || object instanceof Long || object instanceof Float ||
                object instanceof Double || object instanceof Boolean || object instanceof String) {
            return String.valueOf(object);
        }
        return gson.toJson(object);
    }

    /**
     * JSON数据，转成Object
     */
    private <T> T fromJson(String json, Class<T> clazz) {
        return gson.fromJson(json, clazz);
    }


    /**
     * 批量缓存价态
     *
     * @param htDailyrooms List<HtDailyroom>
     * @return 价态对应的日期
     */
    public List<String> saveOrUpdateBatchRates(List<HtDailyRoom> htDailyrooms) {
        if (htDailyrooms.isEmpty()) {
            return new ArrayList<>();
        }
        List<String> dateList = new ArrayList<>();
        redisTemplate.executePipelined((RedisCallback<List>) redisConnection -> {
            for (HtDailyRoom r : htDailyrooms) {
                String date = r.getDate();
                dateList.add(date);
                byte[] key = ("r:" + r.getRpCode() + ":" + date.substring(date.length() - 5).replace("-", "")).getBytes();
                // k存活时间增加一小时
                long diff = DateUtils.suDates(Objects.requireNonNull(DateUtils.stringToDate(date, DateUtils.DATE_PATTERN)), new Date());
                if (diff < 0) {
                    diff = 0;
                }
                long expire = diff + 86400L + 3600L;
                Map<byte[], byte[]> map = new HashMap<>();
                Integer store = r.getSaleNum();
                Integer price = r.getPrice();
                Integer status = r.getStatus();
                Integer subPrice = r.getSubPrice();
                if (store != null) {
                    map.put("n".getBytes(), String.valueOf(store).getBytes());
                }
                if (price != null) {
                    map.put("p".getBytes(), String.valueOf(price).getBytes());
                }
                if (status != null) {
                    map.put("s".getBytes(), String.valueOf(status).getBytes());
                }
                if (subPrice != null) {
                    map.put("b".getBytes(), String.valueOf(subPrice).getBytes());
                }
                map.put("d".getBytes(), date.replaceAll("-", "").getBytes());
                redisConnection.hMSet(key, map);
                redisConnection.expire(key, expire);
            }
            return null;
        });
        return dateList;
    }

    /**
     * 批量获取缓存价态
     *
     * @param dateList 日期集合
     * @param rpCode   产品（rp）编码
     * @return 日期对应价态的集合
     */
    public List<HtDailyRoom> getRatesBatch(List<String> dateList, String rpCode) {
        List<Object> objects = redisTemplate.executePipelined((RedisCallback<List<Object>>) redisConnection -> {

            for (String date : dateList) {
                String key = "r:" + rpCode + ":" + date.substring(date.length() - 5).replace("-", "");
                redisConnection.hMGet(key.getBytes(), "p".getBytes(), "n".getBytes(), "s".getBytes(), "b".getBytes(), "d".getBytes());
            }
            return null;
        });
        List<HtDailyRoom> htDailyrooms = new ArrayList<>();
        for (Object o : objects) {
            List<Integer> list = (List<Integer>) o;
            Integer price = list.get(0);
            Integer num = list.get(1);
            Integer status = list.get(2);
            Integer subPrice = list.get(3);
            Integer date = list.get(4);
            if (Objects.isNull(price)) {
                continue;
            }
            HtDailyRoom htDailyroom = new HtDailyRoom();
            htDailyroom.setSaleNum(num == null ? 0 : num);
            htDailyroom.setPrice(price);
            htDailyroom.setStatus(status == null ? 0 : status);
            htDailyroom.setSubPrice(subPrice);
            htDailyroom.setDate(DateUtils.dateChange(date.toString()));
            htDailyroom.setRpCode(rpCode);
            htDailyrooms.add(htDailyroom);
        }
        return htDailyrooms;
    }

    /**
     * 获取开始日期和结束日期内的价格政策
     *
     * @param start  开始日期
     * @param end    结束日期
     * @param rpCode rp编码
     * @return List<HtDailyRoom>
     */
    public List<HtDailyRoom> getRatesBatch(String start, String end, String rpCode) {
        List<String> dateList = DateUtils.getBetweenDays(start, end);
        return getRatesBatch(dateList, rpCode);
    }

    /**
     * 获取从开始日期 到之后days 日期的价格
     *
     * @param start  开始日期
     * @param days   开始日期之后的天数
     * @param rpCode rp编码
     * @return List<HtDailyRoom>
     */
    public List<HtDailyRoom> getRatesBatch(String start, int days, String rpCode) {
        String end = DateUtils.format(DateUtils.addDateDays(DateUtils.stringToDate(start, DateUtils.DATE_PATTERN), days));
        return getRatesBatch(start, end, rpCode);
    }

    public void upOrDown(String k, String value, Integer type) {
        switch (type) {
            case 0:
                hashOperations.delete(k, value);
                break;
            case -2:
                hashOperations.put(k, value, value);
                break;
            default:
                break;

        }
    }

    public boolean judeDownOrNot(String k, String value) {
        String hotelId = Convert.toStr(hashOperations.get(k, value));
        if (hotelId == null) {
            return false;
        }
        return true;
    }

    public List getHotSaleHotel(String k) {
        JSONArray list = new JSONArray(hashOperations.values(k));
        return list;
    }

    public void HotSaleHotelUpdate(String k, JSONObject hotel, Integer type) {
        switch (type) {
            //删除热销
            case 0:
                hashOperations.delete(k, hotel.get("hotelCode"));
                break;
            //添加热销
            case 1:
                hashOperations.put(k, hotel.getStr("hotelCode"), hotel);
                break;
            default:
                break;

        }
    }

    public JSONObject getSystemSetting(String k, String hashKey) {
        JSONObject setting = new JSONObject(hashOperations.get(k,hashKey));
        return setting;
    }

    public void updateSystemSetting(String k, String hashKey, JSONObject value) {
        hashOperations.put(k, hashKey, value);
    }
}

